(function universalModuleDefinition(root, factory) {
    if (typeof exports === 'object'){
        // CommonJS
        module.exports = factory(require('jquery'));
    } else if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery'], factory);
    } else {
        // Browser
        root.GenerateTrace = factory(root.jQuery);
    }
}(this, function ($) {
    'use strict';

    function GenerateTrace(data) {
        this.constructor(data);
    }

    $.extend(GenerateTrace.prototype = {

        baseUrl: '/api/v1/Utilities/Trace',
        filePath: 'traces_share/',
        intervalHandler: null,
        isBackupsMounted: false,
        sizeDimension: ['B', 'Kb', 'Mb', 'Gb', 'Tb'],
        TRACE_TYPES: {
            IP: 1,
            CUSTOM: 2
        },
        errorMessages: {
            'cannotBeEmpty': WMS._t('rules.CannotBeEmpty'),
            'invalidIpValue': WMS._t('rules.enterValidValue') + ' IP',
            'invalidNameValuer': WMS._t('rules.enterValidValue') + ' [0-9a-z.-_]'
        },

        constructor: function(data) {
            this.isBackupsMounted = data.isBackupsMounted;

            ap_showWaitMessage('generate_trace_main_container', 1);
            var date = new Date();
            var month = (date.getMonth() + 1 > 9) ? date.getMonth() + 1 : '0' + (date.getMonth() + 1);
            var fileName = 'trace_' + date.getFullYear() + '-' + month + '-' + date.getDate() + '_' + date.getHours() + '-' + date.getMinutes() + '.pcap';

            $('#generate_trace_trace_name').val(fileName);

            this._bindEvents();

            if (this._checkIsBackupsMounted()) {
                this.checkStatus();
            } else {
                ap_showWaitMessage('generate_trace_main_container', 0);
            }
        },

        _bindEvents: function() {
            $('#generate_trace_start_btn').on('click', jQuery.proxy(this._onClickStartBtn, this));
            $('#generate_trace_stop_btn').on('click', jQuery.proxy(this._onClickStopBtn, this));
            $('#generate_trace_remove_file').on('click', jQuery.proxy(this._onClickRemoveBtn, this));
            $('#generate_trace_ip_radio, #generate_trace_custom_radio').on('change', jQuery.proxy(this._onChangeRadioBtn, this));
        },

        _showError: function(message) {
            $('#generate_trace_information_fail').show().html(WMS._t('generate_trace.on_failure') + '<br />' + message);

            if(this.intervalHandler){
                clearInterval(this.intervalHandler);
                this.intervalHandler = null;
            }

            $('#generate_trace_information_processing').hide();
            $('#generate_trace_information_processing .size').text('');

            this._switchToStartButtons(true);
            this._unlockInputFields();
        },

        checkStatus: function() {
            var action = 'status';

            $.ajax({
                type: 'GET',
                url: this.baseUrl + '/Status',
                dataType: 'json',
                success: jQuery.proxy(this._onAjaxRequestSuccess, this, action),
                error: jQuery.proxy(this._onAjaxRequestError, this, action)
            });
        },

        showDownloadLink: function(name, size) {
            $('.generate_trace_information').hide();
            $('#generate_trace_information_processing').hide();
            $('#generate_trace_information_processing .size').text('');
            if ($.trim(name) !== '') {
                var title = name;
                if (size && size !== '') {
                    title = title + ' (' + this._reduceSize(size) + ')';
                }
                var downloadLink = '<a title="' + title + '" href="' + this.filePath + name + '" >' + title + '</a>';
                $('#generate_trace_download_trace_link').show();
                $('#generate_trace_link_field').html(downloadLink);
            }

            this.terminateGenerateTrace();
        },

        terminateGenerateTrace: function() {
            if (this.intervalHandler) {
                clearInterval(this.intervalHandler);
                this.intervalHandler = null;
            }

            this._unlockInputFields();
            this._switchToStartButtons(true);
        },

        _checkIsBackupsMounted: function() {
            if (this.isBackupsMounted === false) {
                setError('error', WMS._t('strings.BackupsIsNotMounted'));
                this._lockInputFields();

                return false;
            }

            $('#generate_trace_start_btn').prop('disabled', false);
            $('#generate_trace_stop_btn').prop('disabled', false);

            return true;
        },

        _hideDownloadLink: function() {
            $('#generate_trace_link_field').empty();
            $('#generate_trace_download_trace_link').hide();
        },

        _unlockInputFields: function() {
            if ($('#generate_trace_custom_radio').is(':checked')) {
                $('#generate_trace_custom_tcpdump_input').prop('disabled', false);
                $('#generate_trace_ip_addr').prop('disabled', true);
            } else {
                $('#generate_trace_ip_addr').prop('disabled', false);
                $('#generate_trace_custom_tcpdump_input').prop('disabled', true);
            }

            $('#generate_trace_custom_radio').prop('disabled', false);
            $('#generate_trace_ip_radio').prop('disabled', false);
            $('#generate_trace_trace_name').prop('disabled', false);
        },

        _lockInputFields: function() {
            $('#generate_trace_trace_name').prop('disabled', true);
            $('#generate_trace_ip_radio').prop('disabled', true);
            $('#generate_trace_ip_addr').prop('disabled', true);
            $('#generate_trace_custom_radio').prop('disabled', true);
            $('#generate_trace_custom_tcpdump_input').prop('disabled', true);
        },

        _switchToStartButtons: function(toStart) {
            if (toStart) {
                $('#generate_trace_stop_btn').prop('disabled', true);
                $('#generate_trace_start_btn').prop('disabled', false);
            } else {
                $('#generate_trace_start_btn').prop('disabled', true);
                $('#generate_trace_stop_btn').prop('disabled', false);
            }
        },

        _reduceSize: function(size) {
            var i = 0;
            while (size > 1000) {
                size = size / 1024;
                i++;
            }
            return size.toFixed(1).toString() + this.sizeDimension[i];
        },

        _showErrorMessages: function(errors) {
            for (var name in errors) {
                if (errors.hasOwnProperty(name)) {
                    var errorCode = errors[name].code;
                    var errorMessage = '';

                    switch (errorCode) {
                        case 'WrongRegEx':
                            errorMessage = this.errorMessages.invalidNameValuer;
                            break;
                        case 'InvalidIpAddress':
                            errorMessage = this.errorMessages.invalidIpValue;
                            break;
                        case 'FieldRequired':
                            errorMessage = this.errorMessages.cannotBeEmpty;
                            break;
                        default: break;
                    }

                    this._showHintErrorMessage($('#generate_trace_main_container input[name=' + name + ']'), errorMessage);
                }
            }
        },

        _showHintErrorMessage: function (elem, message) {
            if (!(elem instanceof jQuery) || (typeof message !== 'string') || message === '') {
                return;
            }
            var errorElem = elem.prev();
            if (errorElem.length && errorElem.hasClass('err_span')) {
                errorElem.remove();
            }

            Element.insert(elem[0], {'before': '<span class="err_span" style="color:#ff0000;">#{msg}<br></span>'.interpolate({msg: message})});
        },

        _getFormData: function() {
            var data = {};

            $('#generate_trace_main_container .form_field').each(function (i, el) {
                var fieldName = $(this).attr('name');
                var fieldType = $(this).attr('type');

                if (fieldType === 'radio') {
                    data[fieldName] = $('input[name=' + fieldName + ']:checked', '#generate_trace_main_container').val();
                } else {
                    data[fieldName] = $(this).val();
                }
            });

            return data;
        },

        /************************
         * start events
         ************************/

        _onClickStartBtn: function() {
            if (this.intervalHandler !== null) {
                clearInterval(this.intervalHandler);
            }

            var action = 'start';
            var fieldTypeName = $('#generate_trace_ip_radio').attr('name');
            var data = this._getFormData();

            var options = {
                rules: [ // rule, element_id, message
                    'required,generate_trace_trace_name,' + this.errorMessages.cannotBeEmpty,
                    'reg_exp,generate_trace_trace_name,^[-0-9a-z._]+$,' + this.errorMessages.invalidNameValuer,
                    'if:' + fieldTypeName + '=' + this.TRACE_TYPES.IP + ',valid_ipaddr,generate_trace_ip_addr,' + this.errorMessages.invalidIpValue,
                    'if:' + fieldTypeName + '=' + this.TRACE_TYPES.CUSTOM + ',required,generate_trace_custom_tcpdump_input,' + this.errorMessages.cannotBeEmpty
                ],
                customErrorHandler: function(f, errorInfo) {
                    return errorInfo.collect(function(info) {
                        Element.insert(info[0], {'before': '<span class="err_span" style="color:#ff0000;">#{msg}<br></span>'.interpolate({msg: info[1]})});
                    }).size() === 0;
                }
            };

            if (!WMS.validate('generate_trace_main_container', options)) {
                return;
            }

            if (!$('#generate_trace_download_trace_link').is(':visible') ||
                confirm(WMS._t('generate_trace.confirm.start_generate'))
            ) {
                this._hideDownloadLink();

                $('#generate_trace_main_container .err_span').remove();

                this._switchToStartButtons(false);

                $.ajax({
                    type: 'POST',
                    url: this.baseUrl + '/Start',
                    data: data,
                    dataType: 'json',
                    success: jQuery.proxy(this._onAjaxRequestSuccess, this, action),
                    error: jQuery.proxy(this._onAjaxRequestError, this, action)
                });
            }
        },

        _onChangeRadioBtn: function(event) {
            if (($(event.target).val() | 0) === this.TRACE_TYPES.IP) {
                $('#generate_trace_ip_addr').prop('disabled', false);
                $('#generate_trace_custom_tcpdump_input').prop('disabled', true);
            } else {
                $('#generate_trace_custom_tcpdump_input').prop('disabled', false);
                $('#generate_trace_ip_addr').prop('disabled', true);
            }
        },

        _onClickStopBtn: function() {
            if (this.intervalHandler !== null) {
                clearInterval(this.intervalHandler);
                this.intervalHandler = null;
            }

            var action = 'stop';

            $.ajax({
                type: 'POST',
                url: this.baseUrl + '/Stop',
                dataType: 'json',
                success: jQuery.proxy(this._onAjaxRequestSuccess, this, action),
                error: jQuery.proxy(this._onAjaxRequestError, this, action)
            });
        },

        _onClickRemoveBtn: function() {
            if (confirm(WMS._t('generate_trace.confirm.delete_file'))) {
                // send ajax request to remove file and clear div + hide it
                ap_showWaitMessage('generate_trace_main_container', 1);
                var action = 'remove';

                $.ajax({
                    type: 'POST',
                    url: this.baseUrl + '/Remove',
                    dataType: 'json',
                    success: jQuery.proxy(this._onAjaxRequestSuccess, this, action),
                    error: jQuery.proxy(this._onAjaxRequestError, this, action)
                });
            }
        },

        _onStartSuccess: function() {
            // _lockUnlockButtons
            this._switchToStartButtons(false);
            this._lockInputFields();

            // _showInformation
            $('.generate_trace_information').hide();
            $('#generate_trace_information_processing').show();

            // Checking by interval current status of tcpdump.
            this.intervalHandler = setInterval(jQuery.proxy(this.checkStatus, this), 5 * 1000);
        },

        _onStatusSuccess: function(result) {
            if (result.hasOwnProperty('traceSize')) {
                $('#generate_trace_information_processing .size').text(this._reduceSize(result.traceSize));
            }

            if (result.hasOwnProperty('processIsRunning') && result.processIsRunning === true) {
                $('#generate_trace_information_fail').show().html(' ');

                if (result.traceType === this.TRACE_TYPES.IP) {
                    $('#generate_trace_ip_addr').val(result.traceParam);
                    $('#generate_trace_ip_radio').attr('checked', true);
                } else {
                    $('#generate_trace_custom_tcpdump_input').val(result.traceParam);
                    $('#generate_trace_custom_radio').attr('checked', true);
                }

                $('#generate_trace_trace_name').val(result.traceName);

                if (this.intervalHandler === null) {
                    this._onStartSuccess();
                }
            } else {
                this.showDownloadLink(result.traceName, result.traceSize);
            }

            if (result.hasOwnProperty('traceErrors') && result.traceErrors.length) {
                $('#generate_trace_information_fail').show().html(WMS._t('generate_trace.on_failure') + '<br />' + result.traceErrors.join('<br />'));
            }
        },

        _onStopSuccess: function(result) {
            if(result.hasOwnProperty('processIsRunning') && result.processIsRunning === true){
                // Do nothing
            } else {
                this.showDownloadLink(result.traceName, result.traceSize);
            }
        },

        _onRemoveSuccess: function(result) {
            this._hideDownloadLink();
        },

        _onAjaxRequestSuccess: function(type, response, textStatus, jqXHR) {
            ap_showWaitMessage('generate_trace_main_container', 0);
            if (response.hasOwnProperty('type')) {
                if (response.type === 'result' && response.result !== null) {
                    if (type === 'start') {
                        this._onStartSuccess();
                    } else if (type === 'status') {
                        this._onStatusSuccess(response.result);
                    } else if (type === 'stop') {
                        this._onStopSuccess(response.result);
                    } else if (type === 'remove') {
                        this._onRemoveSuccess(response.result);
                    }
                    return true;
                } else if (response.hasOwnProperty('reason')) {
                    this._onAjaxRequestError(type, jqXHR, textStatus, response.reason);
                } else {
                    this._onAjaxRequestError(type, jqXHR, textStatus, 'Unknown error');
                }
            } else {
                this._onAjaxRequestError(type, jqXHR, textStatus, 'Unknown error');
            }

            return false;
        },

        _onAjaxRequestError: function(type, jqXHR, textStatus, errorThrown) {
            ap_showWaitMessage('generate_trace_main_container', 0);

            if (typeof jqXHR.responseJSON === 'object') {
                var r = jqXHR.responseJSON;
                if (typeof r.errors === 'object') {
                    this._showErrorMessages(r.errors);
                }
                if (r.type === 'error' && typeof r.reason === 'string' && r.reason !== '') {
                    errorThrown += '</br>' + r.reason;
                }
            }
            this._showError(errorThrown);
        }
        /************************
         * end events
         ************************/
    });

    return GenerateTrace;
}));
